import requests

WD_MLserving_TIMEOUT = 2000  # seconds, for long blocking training calls, as needed

API_METHODS_URL = {
    "0.1": {
        "info": "/info",
        "services": "/services",
        "train": "/train",
        "predict": "/predict"
    }
}

class Wanda_MLserving(object):
    """HTTP requests to the Wanda_ML server
    """
    # return types
    RETURN_PYTHON = 0
    RETURN_JSON = 1
    RETURN_NONE = 2
    
    __HTTP = 0
    __HTTPS = 1
    def __init__(self, host="127.0.0.1", port=5412,model_pool_path='predict', proto=0, apiversion="0.1",auth_token=None):
        """ DD class constructor
        Parameters:
        host -- the Wanda_ML server host
        port -- the Wanda_ML server port
        proto -- user http (0,default) or https connection
        """
        self.apiversion = apiversion
        self.__urls = API_METHODS_URL[apiversion]
        self.__host = host
        self.__port = port
        self.__proto = proto
        self.__returntype = self.RETURN_PYTHON
        self.auth_token = auth_token
        self.__model_pool_path=model_pool_path
        if proto == self.__HTTP:
            self.__wdurl = 'http://%s:%d' % (host, port)
        else:
            self.__wdurl = 'https://%s:%d' % (host, port)
    def __getattr__(self, func_name):
        print 'func_name',func_name
        return RemoteFunction(self, func_name)

    def call_func(self, func_name, **kwargs):
        path = func_name
        return self.post(path, **kwargs)
    
    def prepare_headers(self):
        """Prepares headers for sending a request to the Wanda_ML server.
        """
        headers = {}
        headers['Content-type']= "application/json"
        headers['Accept'] = 'application/json'
        if self.auth_token:
            headers['Authorization'] = 'Basic ' + b64encode((self.auth_token['username'] + ':' + self.auth_token['password'])
                                                        .encode('utf-8')).decode('utf-8')
        return headers
    def set_return_format(self, f):
        assert f == self.RETURN_PYTHON or f == self.RETURN_JSON or f == self.RETURN_NONE
        self.__returntype = f
    def set_return_format(self, f):
        assert f == self.RETURN_PYTHON or f == self.RETURN_JSON or f == self.RETURN_NONE
        self.__returntype = f

    def __return_data(self, r):
        if self.__returntype == self.RETURN_PYTHON:
            return r.json()
        elif self.__returntype == self.RETURN_JSON:
            return r.text
        else:
            return None
    def get(self, method, json='machine learning serving is running', params=None):
        """GET to DeepDetect server """
        url = self.__wdurl + method
        r = requests.get(url=url, json=json, params=params, timeout=WD_MLserving_TIMEOUT)
        r.raise_for_status()
        return self.__return_data(r)
    def post(self,_path,**kwargs):
        """POST request to Wanda_ML server"""
        try:
            headers = self.prepare_headers()
            url = self.__wdurl +'/'+self.__model_pool_path+'/'+ _path
            print url
            data, files = self.decouple_files(kwargs)
            #if file:data,if json:json
            r = requests.post(url=url, json=data, headers=headers,timeout=WD_MLserving_TIMEOUT)
            r.raise_for_status()
        except ConnectionError:
            MlservingError('Unable to connect to the server, please try again later.')
        return self.__return_data(r)
    def decouple_files(self, kwargs):
        data = {arg: value for arg, value in kwargs.items() if not self.is_file(value)}
        files = {arg: value for arg, value in kwargs.items() if self.is_file(value)}
        return data, files

    def is_file(self, value):
        return hasattr(value, 'read') or hasattr(value, 'readlines')
    # API methods
    def info(self):
        """Info on the DeepDetect server"""
        return self.get(self.__urls["info"])
def RemoteFunction(client, func_name):
    def wrapped(**kwargs):
        return client.call_func(func_name, **kwargs)
    wrapped.__name__ = func_name
    wrapped.__qualname__ = func_name
    return wrapped
class MlservingError(Exception):
    pass


# -*- coding: utf-8 -*-

from setuptools import setup
from setuptools.command.test import test as TestCommand
import re
import os
import sys

class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = ['tests']
        self.test_suite = True

    def run_tests(self):
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]

def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


version = get_version('rest_wdclient')
if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    os.system("python setup.py bdist_wheel upload")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()


setup(
    name='wdmlservice',
    version=version,
    url='https://github.com/leepand',
    license='BSD',
    description='REST APIs for wanda machine learning service, made easy.',
    author='Li Pandeng',
    author_email='pandeng.li@163.com',  # SEE NOTE BELOW (*)
    packages=get_packages('rest_wdclient'),
    package_data=get_package_data('rest_wdclient'),
    cmdclass={'test': PyTest},
    install_requires=['requests']
    
)
